@@ -23,37 +23,37 @@ Test::System - Test suite oriented for testing system administration tasks
 =head1 DESCRIPTION
 
 Loads and runs a set of tests for verifying a system administration task
-before a possible incident or after a task gets executed.
+before a possible incident happens or after a task is executed.
 
-Tests can be run as perl scripts and actually in any format I<TAP::Harness>
+Tests can be run as perl scripts and actually in any format L<TAP::Harness>
 supports.
 
-Some examples of how I<Test::System> can help you:
+Some examples of how L<Test::System> can help you:
 
 =over 4
 
 =item *
 
 Before a deployment probably you want to make sure your systems and
-configuration files are ready to go. Also you probably also want to make sure
-that if you needed to apply patches to your configuration files that they
+configuration files are ready to go. Also you probably want to make sure
+that if you needed to apply patches to your configuration files, they
 really got applied (even in a development server or verify directly on
-production servers). So here I<Test::System> can help you verifying this
-in the form of a test case.
+production servers). So here L<Test::System> can help you verifying this
+in the form of a test case. Of course, you write the test case :-)
 
 =item *
 
 What about if you have many servers and although you have nagios to verify
-the CPU of the speed sometimes you need to verify it "on-demand" (perhaps when
-you get new servers in your DC). Here I<Test::System> can help you to verify
-these servers are ready to go before even they get connected to nagios or your
-favorite monitoring tool.
+the speed of the CPU sometimes you need to verify it "on-demand" (perhaps when
+you get new servers in your datacenter). Here L<Test::System> can help you to
+verify that these servers are ready to go before even they get connected to nagios
+or your favorite monitoring tool.
 
 =back
 
-One of the things you need to keep in mind is that I<Test::System> is B<NOT> a
+One of the things you need to keep in mind is that L<Test::System> is B<NOT> a
 monitoring system. It will let you verify things on demand, things that are
-repetitive and perhaps you want to have them automated.
+repetitive and perhaps you want them to be automated.
 
 =cut
 use 5.006;
@@ -65,18 +65,17 @@ use Test::System::Output::Factory;
 use TAP::Harness;
 use UNIVERSAL qw(isa);
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 =head1 Attributes
 
-The module offers a list of attributes that some only have read only access
-and others allow write access.
+The module offers a list of attributes, some of them are read-only.
 
 =over 6
 
 =item B<test_groups>
 
-YAML filename of where a list of available tests are. This is not required but
+YAML filename that has a list of available tests. This is not required but
 can be useful if you want to group tests (like I<hardware.yaml>, I<net.yaml>,
 etc). Comes also handy when user does not provide a list of tests to execute,
 so all the tests listed in this file are executed.
@@ -114,7 +113,7 @@ Is an attribute that can be represented as a string (like a hostname) or as a
 list (where each item will be a node/hostname). This attribute has write access
 and is where the tests are going to be executed to.
 
-Another way of setting this value is when C<runtests()> is called a test plan
+Another way of setting this value is when C<runtests()> is called, a test plan
 (in the form of a YAML file) can be provided and it can contain the list of
 nodes to use.
 
@@ -164,7 +163,7 @@ has 'custom_factory' => (
 
 =item B<harness>
 
-Is (or will be) the harness instance once runtests gets called.
+Is (or will be) the harness instance once C<runtests> is executed.
 
 =cut
 has 'harness' => (
@@ -178,15 +177,16 @@ the items of this hash to environment variables.
 
 The use of this attribute is very handy if you want to provide some additional
 data for your tests and since the tests are run in separate forks with
-Test::Harness then the only possible way to keep them is to make them available
-through the environment (C<%ENV> hash).
+L<Test::Harness> then the only possible way to keep them is to make them
+available through the environment (C<%ENV> hash).
 
 Please be warned that only scalars are stored in environment variables, those
 that are an array will be converted to CSV values while the rest of the data
 types will be lost.
 
 In your tests if you want to use any of these parameters they will be available
-through the environment variables with a prefix of: C<TEST_SYSTEM_>.
+through the environment variables with a prefix of: C<TEST_SYSTEM_> or you
+can use L<Test::System::Helper> to get their values.
 
 =cut
 has 'parameters' => (
@@ -218,8 +218,8 @@ has 'results' => (
 
 =item B<status>
 
-A string (read-only) that contains a word describing the status of all tests,
-this value is also generated by L<TAP::Parser::Aggregator>.
+A string (read-only) that contains a word that describes the status of all
+the tests. This value is also generated by L<TAP::Parser::Aggregator>.
 
 =back
 
@@ -231,12 +231,22 @@ has 'status' => (
 
 =head1 Methods
 
-=head2 B<runtests( @tests , %options )>
+=head2 B<runtests(@tests,%options)>
 
-It will run a group of given test cases, however if no list is given or is
-empty then all the available cases will be run.
+It will run the given test cases (C<@tests>). The C<@tests> can be:
 
-The C<%options> is a hash of options that will be passed to the I<TAP::Harness>
+=over 4
+
+=item * An array of test files to execute.
+
+=item * Or, a string pointing to a test plan file (YAML).
+
+=back
+
+If no C<@tests> are given but we have a C<test_groups> then B<ALL> the tests
+listed inside this file will be executed.
+
+The C<%options> is a hash of options that will be passed to the L<TAP::Harness>
 object, some useful parameters are:
 
 =over 4
@@ -274,40 +284,39 @@ sub runtests {
     if (!@tests_to_run and $self->yaml_file) {
         @tests_to_run = keys(%{$self->available_tests});
     }
+
+    $self->pretests_verification();
     if (!$self->nodes) {
         die "No nodes were specified";
     }
     # No duplicate tests and build the module name
     my (%seen, @test_files);
     foreach (@tests_to_run) {
-        if (!$seen{$_}++) {
-            # Does the test exist?
-            my ($code, $description) = ($_, $_);
-            if (defined $self->available_tests) {
-                if (defined $self->available_tests->{$_}) {
-                    if (defined $self->available_tests->{$_}->{'code'}) {
-                        $code = $self->available_tests->{$_}->{'code'};
-                    }
-                    if (defined $self->available_tests->{$_}->{'description'}) {
-                        $description = $self->available_tests->{$_}->{'description'};
-                    }
+        # Does the test exist?
+        my ($code, $description) = ($_, $_);
+        if (defined $self->available_tests) {
+            if (defined $self->available_tests->{$_}) {
+                if (defined $self->available_tests->{$_}->{'code'}) {
+                    $code = $self->available_tests->{$_}->{'code'};
                 }
-                if (!-f $code) {
-                    warn "$_ was not found inside the test groups file, skipping";
-                    next;
-                }
-            } else {
-                if (!-f $_) {
-                    warn "$_ was not found on a test YAML file or as a file, skipping";
-                    next;
+                if (defined $self->available_tests->{$_}->{'description'}) {
+                    $description = $self->available_tests->{$_}->{'description'};
                 }
             }
-            if ($code and $description) {
-                push(@test_files, [ $code, $description ]);
+            if (!-f $code) {
+                warn "$_ was not found inside the test groups file, skipping";
+                next;
+            }
+        } else {
+            if (!-f $_) {
+                warn "$_ was not found on a test YAML file or as a file, skipping";
+                next;
             }
         }
+        if ($code and $description) {
+            push(@test_files, [ $code, $description ]);
+        }
     }
-    $self->prepare_environment();
     my $factory_class = 'Test::System::Output::Factory';
     if ($self->custom_factory) {
         $factory_class = $self->custom_factory;
@@ -331,6 +340,7 @@ sub runtests {
         }
     }
     $options->{'lib'} = \@lib;
+    $self->prepare_environment(@test_files, $options);
     $self->{harness} = TAP::Harness->new($options);
     $self->{harness}->callback('after_runtests' =>
             sub { $self->set_results(shift) });
@@ -358,6 +368,22 @@ sub setup_parser {
     $parser->callback('ALL' => sub { $self->what_happened($parser, shift) });
 }
 
+=head2 B<pretests_verification(@tests, %options)>
+
+Does some verification before the tests are executed. This method gets called
+by C<runtests>.
+
+The parameters it accepts are the same parameters passed to C<runtests> with
+the main difference the tests are already filtered (eg, if a test plan in
+YAML was provided we will use it or we will use all the tests listed inside
+the C<test_groups>.
+
+=cut
+sub pretests_verification {
+    my ($selv, $tests, $options) = @_;
+    }
+
+
 =head2 B<what_happened($parser, $result_test)>
 
 Similar to I<setup_parser>, this should never be called since it will be
@@ -400,8 +426,8 @@ sub what_happened {
 
 =head2 B<set_results($aggregator)>
 
-This is a callback and is only called/triggered by I<TAP::Harness> when all the
-tests are done. It will fill the I<results> attribute with information of
+This is a callback and is only called/triggered by L<TAP::Harness> when all the
+tests are done. It will fill the C<results> attribute with information of
 everything.
 
 Refer to the documentation of I<results> for more information.
@@ -445,7 +471,7 @@ sub set_results {
     }
 }
 
-=head2 B<prepare_environment ()>
+=head2 B<prepare_environment()>
 
 Prepares the environment by settings the needed environment values so they can
 be used later by the tests
@@ -473,7 +499,7 @@ sub prepare_environment {
     }
 }
 
-=head2 B<clean_environment ()>
+=head2 B<clean_environment()>
 
 Cleans/deletes all the environment variables that match C<TEST_SYSTEM_*>
 
@@ -625,6 +651,10 @@ sub _generate_output_types_hash {
     return \%hash;
 }
 
+=head1 SEE ALSO
+
+Take a look to an awesome and pretty similar CPAN module: L<Test::Server>.
+
 =head1 AUTHOR
  
 Pablo Fischer (pablo@pablo.com.mx).
